__title__ = "fastcrc"
__description__ = "A hyper-fast Python module for computing CRC(16, 32, 64) checksum"
__url__ = "https://github.com/overcat/fastcrc"
__issues__ = f"{__url__}/issues"
__version__ = "0.2.1-dev.2"
__author__ = "overcat"
__author_email__ = "4catcode@gmail.com"
__license__ = "MIT License"
